/**
 * Copyright 2012 the project-owners
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package inti.ws.spring.resource.files;

import inti.ws.spring.resource.ByteWebResource;
import inti.ws.spring.resource.WebResource;
import inti.ws.spring.resource.config.ConfigParser;
import inti.ws.spring.resource.config.ConfigParserSettings;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.inject.Inject;
import javax.servlet.ServletContext;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;

@Component
@Scope("prototype")
public class FilesConfig implements ConfigParser<WebResource> {

	@Inject
	ServletContext ctx;
	protected ConfigParserSettings settings;

	@Override
	public ConfigParserSettings getConfigParserSettings() {
		return settings;
	}

	@Override
	public void setConfigParserSettings(ConfigParserSettings settings) {
		this.settings = settings;
	}

	@Override
	public Map<String, WebResource> instanceWebResources(ObjectMapper mapper, JsonNode node) throws Exception {
		Map<String, WebResource> files = new HashMap<String, WebResource>();
		Iterator<String> iter;
		String name;

		if (node != null) {

			iter = node.fieldNames();

			if (iter != null) {

				while (iter.hasNext()) {
					name = iter.next();

					files.put(name, new ByteWebResource(ctx, node.get(name).asText()));
				}

			}

		}

		return files;
	}

	@Override
	public void configureWebResources(ObjectMapper mapper, JsonNode node, Map<String, WebResource> resources,
	        Map<String, Map<String, WebResource>> resourceStack) throws Exception {
	}

}
